@@ -12,9 +12,9 @@ import platform
 import inspect
 import pydoc
 import xmlrpclib
-from SimpleXMLRPCServer import SimpleXMLRPCDispatcher
-from jsonrpcdispatcher import JSONRPCDispatcher, json
 from xmlrpclib import Fault
+from jsonrpcdispatcher import JSONRPCDispatcher, json
+from xmlrpcdispatcher import XMLRPCDispatcher
 
 # this error code is taken from xmlrpc-epi
 # http://xmlrpc-epi.sourceforge.net/specs/rfc.fault_codes.php
@@ -156,14 +156,7 @@ class RPCDispatcher:
         self.url = url
         self.rpcmethods = []        # a list of RPCMethod objects
         self.jsonrpcdispatcher = JSONRPCDispatcher()
-        
-        if int(version[0]) < 3 and int(version[1]) < 5:
-            # this is for python 2.4 and below
-            self.xmlrpcdispatcher = SimpleXMLRPCDispatcher()
-        else:
-            # python 2.5+ requires different parameters
-            self.xmlrpcdispatcher = SimpleXMLRPCDispatcher(allow_none=False, 
-                                                           encoding=None)
+        self.xmlrpcdispatcher = XMLRPCDispatcher()
             
         if not restrict_introspection:
             self.register_method(self.system_listmethods)
@@ -245,19 +238,19 @@ class RPCDispatcher:
                     # decorator, add it to the dispatcher
                     self.register_method(method, method.external_name)     
     
-    def jsondispatch(self, raw_post_data):
+    def jsondispatch(self, raw_post_data, **kwargs):
         '''
         Sends the post data to a jsonrpc processor
         '''
         
-        return self.jsonrpcdispatcher.dispatch(raw_post_data)
+        return self.jsonrpcdispatcher.dispatch(raw_post_data, **kwargs)
     
-    def xmldispatch(self, raw_post_data):
+    def xmldispatch(self, raw_post_data, **kwargs):
         '''
         Sends the post data to an xmlrpc processor
         '''
         
-        return self.xmlrpcdispatcher._marshaled_dispatch(raw_post_data)
+        return self.xmlrpcdispatcher.dispatch(raw_post_data, **kwargs)
         
     def get_method_name(self, raw_post_data, request_format='xml'):
         '''
