@@ -6,12 +6,15 @@ RPC Dispatcher Tests
 
 '''
 
+import base64
 import unittest
-from xmlrpclib import Fault
+from xmlrpclib import Fault, Binary
 from xml.dom.minidom import parseString
 from rpc4django.rpcdispatcher import *
 from rpc4django.jsonrpcdispatcher import *
 
+BINARY_STRING = '\x97\xd2\xab\xc8\xfc\x98\xad'
+
 # tests both the class and the decorator
 class TestRPCMethod(unittest.TestCase):
     def setUp(self):
@@ -50,6 +53,17 @@ class TestRPCDispatcher(unittest.TestCase):
         
         self.add = add
         
+        def kwargstest(a, b, **kwargs):
+            if kwargs.get('c', None) is not None:
+                return True
+            return False
+        
+        self.kwargstest = kwargstest
+        
+        def testBin():
+            return Binary(BINARY_STRING)
+        self.testBin = testBin
+        
     def test_rpcfault(self):
         try:
             resp = self.d.system_methodhelp('method.DoesNotExist.AtAll')
@@ -98,6 +112,15 @@ class TestRPCDispatcher(unittest.TestCase):
         retval = dom.getElementsByTagName('string')[0].firstChild.data
         self.assertEqual(retval, u'はじめまして')
         
+    def test_base64_call(self):
+        self.d.register_method(self.testBin)
+        
+        xml = xml = '<?xml version="1.0"?><methodCall><methodName>testBin</methodName><params></params></methodCall>'
+        resp = self.d.xmldispatch(xml)
+        dom = parseString(resp)
+        retval = dom.getElementsByTagName('base64')[0].firstChild.data
+        self.assertEquals(base64.b64decode(retval), BINARY_STRING)
+        
     def test_jsonrpc_call(self):
         jsontxt = '{"params":[],"method":"system.listMethods","id":1}'
         resp = self.d.jsondispatch(jsontxt)
@@ -115,6 +138,18 @@ class TestRPCDispatcher(unittest.TestCase):
         self.assertTrue(jsondict['error'] is None)
         self.assertEqual(jsondict['id'], 1)
         self.assertEqual(jsondict['result'], 3)
+        
+    def test_kwargs(self):
+        self.d.register_method(self.kwargstest)
+        
+        jsontxt = '{"params":[1,2],"method":"kwargstest","id":1}'
+        resp = self.d.jsondispatch(jsontxt)
+        jsondict = json.loads(resp)
+        self.assertFalse(jsondict['result'])
+        
+        resp = self.d.jsondispatch(jsontxt, c=1)
+        jsondict = json.loads(resp)
+        self.assertTrue(jsondict['result'])
 
 if __name__ == '__main__':
     unittest.main()
