@@ -33,6 +33,11 @@ RESTRICT_XML = getattr(settings, 'RPC4DJANGO_RESTRICT_XMLRPC', False)
 RESTRICT_METHOD_SUMMARY = getattr(settings, 
                                   'RPC4DJANGO_RESTRICT_METHOD_SUMMARY', False)
 RESTRICT_RPCTEST = getattr(settings, 'RPC4DJANGO_RESTRICT_RPCTEST', False)
+RESTRICT_RPCTEST = getattr(settings, 'RPC4DJANGO_RESTRICT_RPCTEST', False)
+HTTP_ACCESS_CREDENTIALS = getattr(settings, 
+                                  'RPC4DJANGO_HTTP_ACCESS_CREDENTIALS', False)
+HTTP_ACCESS_ALLOW_ORIGIN = getattr(settings, 
+                                  'RPC4DJANGO_HTTP_ACCESS_ALLOW_ORIGIN', '')
 
 # get a list of the installed django applications
 # these will be scanned for @rpcmethod decorators
@@ -124,7 +129,6 @@ def serve_rpc_request(request):
     '''
     This method handles rpc calls based on the content type of the request
     '''
-    
 
     if request.method == "POST" and len(request.POST) > 0:
         # Handle POST request with RPC payload
@@ -139,7 +143,8 @@ def serve_rpc_request(request):
             if not _check_request_permission(request, 'xml'):
                 return HttpResponseForbidden()
             
-            resp = dispatcher.xmldispatch(request.raw_post_data)
+            resp = dispatcher.xmldispatch(request.raw_post_data, \
+                                          request=request)
             response_type = 'text/xml'
         else:
             if RESTRICT_JSON:
@@ -148,13 +153,34 @@ def serve_rpc_request(request):
             if not _check_request_permission(request, 'json'):
                 return HttpResponseForbidden()
             
-            resp = dispatcher.jsondispatch(request.raw_post_data)
+            resp = dispatcher.jsondispatch(request.raw_post_data, \
+                                           request=request)
             response_type = 'application/json'
             
         if LOG_REQUESTS_RESPONSES:
             logging.debug('Outgoing %s response: %s' %(response_type, resp))
         
         return HttpResponse(resp, response_type)
+    elif request.method == 'OPTIONS':
+        # Handle OPTIONS request for "preflighted" requests
+        # see https://developer.mozilla.org/en/HTTP_access_control
+        
+        response = HttpResponse('', 'text/plain')
+        
+        origin = request.META.get('HTTP_ORIGIN', 'unknown origin')
+        response['Access-Control-Allow-Methods'] = 'POST, GET, OPTIONS'
+        response['Access-Control-Max-Age'] = 0
+        response['Access-Control-Allow-Credentials'] = \
+                        str(HTTP_ACCESS_CREDENTIALS).lower()
+        response['Access-Control-Allow-Origin']= HTTP_ACCESS_ALLOW_ORIGIN
+        
+        response['Access-Control-Allow-Headers'] = \
+                    request.META.get('HTTP_ACCESS_CONTROL_REQUEST_HEADERS', '')
+                    
+        if LOG_REQUESTS_RESPONSES:
+            logging.debug('Outgoing HTTP access response to: %s' %(origin))
+                    
+        return response
     else:
         # Handle GET request
         
@@ -183,6 +209,22 @@ try:
 except NoReverseMatch:
     URL = ''
     
+# exclude from the CSRF framework because RPC is intended to be used cross site
+try:
+    # Django 1.2
+    from django.views.decorators.csrf import csrf_exempt
+except ImportError:
+    try:
+        # Django 1.1
+        from django.contrib.csrf.middleware import csrf_exempt
+    except ImportError:
+        # Django 1.0
+        csrf_exempt = None
+
+if csrf_exempt is not None:
+    csrf_exempt(serve_rpc_request)
+
+    
 # instantiate the rpcdispatcher -- this examines the INSTALLED_APPS
 # for any @rpcmethod decorators and adds them to the callable methods
 dispatcher = RPCDispatcher(URL, APPS, RESTRICT_INTROSPECTION) 
